--[[
Copyright (c) 2011 WangXH <kurapica.igas@gmail.com>

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
--]]

-- Class System
-- Author: kurapica.igas@gmail.com
-- Create Date : 2011/02/01
-- ChangeLog   :
--               2011/04/14	System.Reflector added.
--               2011/05/11 Script handlers can run as thread.
--				 2011/05/30 System.Reflector.GetParts added.
--               2011/06/24 Property definition ignore case
--               2011/07/04 Struct no longer need Constructor and Validate function
--               2011/07/08 System.Reflector.IsAbstractClass(cls) added
--               2011/07/12 System.Reflector.ParseEnum(enm, value) added
--               2011/07/27 Enum's validation method changed.
--               2011/10/26 Error report update for __index & __newindex
--               2011/10/30 System.Reflector.ConvertClass(obj, cls) added
--               2011/10/31 System.Reflector.InactiveThread added
--               2012/04/10 System.Reflector.IsNameSpace(obj) added
------------------------------------------------------------------------
-- Class system is used to provide a object-oriented system in lua.
-- With this system, you can created a class like
--
-- namespace "System"				-- define the namespace
--
-- class "MyClass"						-- declare starting to define the class
--		inherit "Object"					-- declare the class is inherited from System.Object
--
--		script "OnNameChanged"	-- declare the class have a script named "OnNameChanged"
--
--		function Print(self)				-- the global functions will be treated as the class's methodes, self means the object
--			print(self._Name)
--		end
--
--		property "Name" {				-- declare the class have a property named "Name"
--			Get = function(self)		-- the get method for property "Name"
--				return self._Name
--			end,
--			Set = function(self, value)	-- the set method for property "Name"
--				self._Name = value
--				self:OnNameChanged(value)	-- raise the "OnNameChanged" script to trigger it's handler functions.
--			end,
--			Type = String,				-- the property "Name"'s type, so when you assign a value to Name, it should be checked.
--		}
--
--		function MyClass(name)		-- the function with same name of the class is treated as the Constructor of the class
--			local obj = {_Name = name}	-- the object should be a table
--			return obj
--		end
--	endclass "MyClass"				-- declare the definition of the class is over.
--
--	Using MyClass:
--
--	myObj = MyClass("Test")
--
--	myObj:Print()						-- print out : Test
--
--	function myObj:OnNameChanged(name)	-- define the script handler for 'OnNameChanged'
--		print("The Name is changed to "..name)
--	end
--
--	myObj.Name = "Hello"			-- print out : The Name is changed to Hello
------------------------------------------------------------------------

local version = 19

------------------------------------------------------
-- Version Check & Class Environment
------------------------------------------------------
do
	local _Meta = getmetatable(IGAS)

	-- Version Check
	if _Meta._IGAS_Class_Version and _Meta._IGAS_Class_Version >= version then
		return
	end

	_Meta._IGAS_Class_Version = version

	-- Class Environment
	_Meta._Class_ENV = _Meta._Class_ENV or {}
	_Meta._Class_ENV._Class_KeyWords = _Meta._Class_KeyWords

	setmetatable(_Meta._Class_ENV, {
		__index = function(self,  key)
			if type(key) == "string" and key ~= "_G" and key:find("^_") then
				return
			end

			if _G[key] then
				rawset(self, key, _G[key])
				return rawget(self, key)
			end
		end,
	})

	setfenv(1, _Meta._Class_ENV)

	-- Scripts
	strtrim = strtrim or function(s)
	  return s and (s:gsub("^%s*(.-)%s*$", "%1")) or ""
	end

	wipe = wipe or function(t)
		for k in pairs(t) do
			t[k] = nil
		end
	end

	geterrorhandler = geterrorhandler or function()
		return print
	end

	errorhandler = errorhandler or function(err)
		return geterrorhandler()(err)
	end

	tinsert = tinsert or table.insert
	tremove = tremove or table.remove
	unpack = unpack
	pcall = pcall
	sort = sort

	TYPE_CLASS = "Class"
	TYPE_ENUM = "Enum"
	TYPE_STRUCT = "Struct"

	TYPE_NAMESPACE = "IGAS.NameSpace"
	TYPE_TYPE = "IGAS.TYPE"
end

------------------------------------------------------
-- NameSpace
------------------------------------------------------
do
	_NameSpace = _NameSpace or newproxy(true)

	_NSInfo = _NSInfo or setmetatable({}, {
		__index = function(self, key)
			if not IsNameSpace(key) then
				return
			end

			self[key] = {
				Owner = key,
			}

			return rawget(self, key)
		end,
	})

	-- metatable for class
	_MetaNS = _MetaNS or getmetatable(_NameSpace)
	do
		_MetaNS.__call = function(self, ...)
			local info = _NSInfo[self]

			if info.Type == TYPE_CLASS then
				return Class2Obj(self, ...)
			elseif info.Type == TYPE_STRUCT then
				return Struct2Obj(self, ...)
			end

			error(("%s can't be used as a constructor."):format(tostring(self)), 2)
		end

		_MetaNS.__index = function(self, key)
			local info = _NSInfo[self]

			if info.Type == TYPE_STRUCT then
				return info.SubNS and info.SubNS[key]
			elseif info.Type == TYPE_CLASS then
				if info.SubNS and info.SubNS[key] then
					return info.SubNS[key]
				elseif type(key) == "string" and (not key:find("^_")) and info.Cache4Method and info.Cache4Method[key] then
					return info.Cache4Method[key]
				elseif _KeyMeta[key] ~= nil then
					if _KeyMeta[key] then
						return info.MetaTable[key]
					else
						return info.MetaTable["_"..key]
					end
				end
			elseif info.Type == TYPE_ENUM then
				return type(key) == "string" and info.Enum[key:upper()] or error(("%s is not an enumeration value of %s."):format(tostring(key), tostring(self)), 2)
			else
				return info.SubNS and info.SubNS[key]
			end
		end

		_MetaNS.__newindex = function(self, key, value)
			error(("can't set value for %s, it's readonly."):format(tostring(self)), 2)
		end

		_MetaNS.__add = function(v1, v2)
			local ok, _type1, _type2

			ok, _type1 = pcall(BuildType, v1)
			if not ok then
				_type1 = strtrim(_type1:match(":%d+:(.*)$") or _type1)
				error(_type1, 2)
			end

			ok, _type2 = pcall(BuildType, v2)
			if not ok then
				_type2 = strtrim(_type2:match(":%d+:(.*)$") or _type2)
				error(_type2, 2)
			end

			return _type1 + _type2
		end

		_MetaNS.__tostring = function(self)
			return GetFullName4NS(self)
		end

		_MetaNS.__metatable = TYPE_NAMESPACE
	end

	-- IsNameSpace
	function IsNameSpace(ns)
		return ns and type(ns) == "userdata" and getmetatable(ns) == TYPE_NAMESPACE or false
	end

	-- BuildNameSpace
	function BuildNameSpace(ns, namelist)
		if type(namelist) ~= "string" or (ns ~= nil and not IsNameSpace(ns)) then
			return
		end

		if namelist:find("%.%s*%.") then
			error("the namespace 's name can't have empty string between dots.", 2)
		end

		local cls = ns
		local info = cls and _NSInfo[cls]
		local parent = cls

		for name in namelist:gmatch("[^%.]+") do
			name = name:match("[_%w]+")

			if not name or name == "" then
				error("the namespace's name must be composed with number, string or '_'.", 2)
			end

			if not info then
				cls = newproxy(_NameSpace)
			elseif info.Type == nil or info.Type == TYPE_CLASS or info.Type == TYPE_STRUCT then
				info.SubNS = info.SubNS or {}
				info.SubNS[name] = info.SubNS[name] or newproxy(_NameSpace)

				cls = info.SubNS[name]
			else
				error(("can't add item to a %s."):format(tostring(info.Type)), 2)
			end

			info = _NSInfo[cls]
			info.Name = name
			if not info.NameSpace and parent ~= _NameSpace then
				info.NameSpace = parent
			end
			parent = cls
		end

		if cls == ns then
			return
		end

		return cls
	end

	-- GetNameSpace
	function GetNameSpace(ns, namelist)
		if type(namelist) ~= "string" or not IsNameSpace(ns) then
			return
		end

		if namelist:find("%.%s*%.") then
			error("the namespace 's name can't have empty string between dots.", 2)
		end

		local cls = ns

		for name in namelist:gmatch("[^%.]+") do
			name = name:match("[_%w]+")

			if not name or name == "" then
				error("the namespace's name must be composed with number, string or '_'.", 2)
			end

			cls = cls[name]
			if not cls then
				return
			end
		end

		if cls == ns then
			return
		end

		return cls
	end

	-- GetDefaultNameSpace
	function GetDefaultNameSpace()
		return _NameSpace
	end

	-- SetNameSpace
	function SetNameSpace4Env(env, name)
		if type(env) ~= "table" then
			return
		end

		if type(name) == "string" then
			local ns = BuildNameSpace(GetDefaultNameSpace(), name)

			if ns then
				rawset(env, "__IGAS_NameSpace", ns)
			else
				rawset(env, "__IGAS_NameSpace", nil)
			end
		elseif IsNameSpace(name) then
			rawset(env, "__IGAS_NameSpace", name)
		else
			rawset(env, "__IGAS_NameSpace", nil)
		end
	end

	-- GetEnvNameSpace
	function GetNameSpace4Env(env)
		local ns = type(env) == "table" and env.__IGAS_NameSpace
		if IsNameSpace(ns) then
			return ns
		end
	end

	-- GetFullName4NS
	function GetFullName4NS(ns)
		local info = _NSInfo[ns]

		if info then
			local name = info.Name

			while info and info.NameSpace do
				info = _NSInfo[info.NameSpace]

				if info then
					name = info.Name.."."..name
				end
			end

			return name
		end
	end

	------------------------------------
	--- Set the default namespace for the current environment, the class defined in this environment will be stored in this namespace
	-- @name namespace
	-- @class function
	-- @param name the namespace's name list, using "." to split.
	-- @usage namespace "Widget"
	------------------------------------
	function namespace(name)
		if name ~= nil and type(name) ~= "string" and not IsNameSpace(name) then
			error([[Usage: namespace "namespace"]], 2)
		end

		local fenv = getfenv(2)

		SetNameSpace4Env(fenv, name)
	end
end

------------------------------------------------------
-- Type
------------------------------------------------------
do
	_Meta4Type = _Meta4Type or {}
	do
		_Meta4Type.__add = function(v1, v2)
			local ok, _type1, _type2

			ok, _type1 = pcall(BuildType, v1)
			if not ok then
				_type1 = strtrim(_type1:match(":%d+:(.*)$") or _type1)
				error(_type1, 2)
			end

			ok, _type2 = pcall(BuildType, v2)
			if not ok then
				_type2 = strtrim(_type2:match(":%d+:(.*)$") or _type2)
				error(_type2, 2)
			end

			if _type1 and _type2 then
				local _type = setmetatable({}, _Meta4Type)

				_type.AllowNil = _type1.AllowNil or _type2.AllowNil

				local tmp = {}

				for _, ns in ipairs(_type1) do
					tinsert(_type, ns)
					tmp[ns] = true
				end
				for _, ns in ipairs(_type2) do
					if not tmp[ns] then
						tinsert(_type, ns)
					end
				end

				tmp = nil

				return _type
			else
				return _type1 or _type2
			end
		end

		_Meta4Type.__index = {
			-- value = Type:Validate(value)
			Validate = function(self, value)
				if value == nil and self.AllowNil then
					return value
				end

				local flag, msg, info, new

				for _, ns in ipairs(self) do
					info = _NSInfo[ns]

					if not info then
						-- do nothing
					elseif info.Type == TYPE_CLASS then
						-- Check if the value is an instance of this class
						if type(value) == "table" and getmetatable(value) and getmetatable(value).__class and IsChildClass(ns, getmetatable(value).__class) then
							return value
						else
							msg = msg or ("%s must be an instance of class '%s'."):format("%s", tostring(ns))
						end
					elseif info.Type == TYPE_ENUM then
						-- Check if the value is an enumeration value of this enum
						if type(value) == "string" and info.Enum[value:upper()] then
							return info.Enum[value:upper()]
						end

						for _, v in pairs(info.Enum) do
							if value == v then
								return v
							end
						end

						msg = msg or ("%s must be a value of enum %s."):format("%s", tostring(ns))
					elseif info.Type == TYPE_STRUCT then
						-- Check if the value is an enumeration value of this structure
						flag, new = pcall(ValidateStruct, ns, value)

						if flag then
							return new
						elseif not msg then
							new = strtrim(new:match(":%d+:(.*)$") or new)

							if new:find("%%s([_%w]+)") then
								msg = new:gsub("%%s", "%%s"..self.Name..".")
							else
								msg = new:gsub("%%s", "%%s"..self.Name)
							end
						end
					end
				end

				assert(not msg, msg)

				return value
			end,

			-- newType = type:Copy()
			Copy = function(self)
				local _type = setmetatable({}, _Meta4Type)

				for i, v in pairs(self) do
					_type[i] = v
				end

				return _type
			end,

			-- boolean = type:Is(nil)
			-- boolean = type:Is(System.String)
			Is = function(self, ns)
				local fenv = getfenv(2)

				if ns == nil then
					return self.AllowNil or false
				end

				if IsNameSpace(ns) then
					for _, v in ipairs(self) do
						if v == ns then
							return true
						end
					end
				end

				return false
			end,
		}

		_Meta4Type.__metatable = TYPE_TYPE
	end

	function IsType(tbl)
		return type(tbl) == "table" and getmetatable(tbl) == TYPE_TYPE or false
	end

	function BuildType(ns, name)
		local allowNil = false

		if ns == nil then
			allowNil = true
		elseif IsType(ns) then
			if name then
				ns.Name = name
			end
			return ns
		end

		if ns == nil or IsNameSpace(ns) then
			local _type = setmetatable({}, _Meta4Type)

			_type.AllowNil = allowNil or nil
			tinsert(_type, ns)

			if name then
				_type.Name = name
			end
			return _type
		else
			error("The Type must be nil, struct, enum or class.")
		end
	end
end

------------------------------------------------------
-- Class
------------------------------------------------------
do
	_ClsEnv2Info = _ClsEnv2Info or setmetatable({}, {__mode = "kv",})
	_ClsCache2Info = _ClsCache2Info  or setmetatable({}, {__mode = "kv",})
	_ClsMeta2Info = _ClsMeta2Info or  setmetatable({}, {__mode = "kv",})

	_KeyWord4ClsEnv = _KeyWord4ClsEnv or {}

	_KeyMeta = {
		__add = true,		-- a + b
		__sub = true,		-- a - b
		__mul = true,		-- a * b
		__div = true,		-- a / b
		__mod = true,		-- a % b
		__pow = true,		-- a ^ b
		__unm = true,		-- - a
		__concat = true,	-- a..b
		__len = true,		-- #a
		__eq = true,		-- a == b
		__lt = true,		-- a < b
		__le = true,		-- a <= b
		__index = false,	-- return a[b]
		__newindex = false,	-- a[b] = v
		__call = true,		-- a()
		__gc = false,		-- dispose a
		__tostring = true,	-- tostring(a)
		__exist = true,		-- ClassName(...)	-- return object if existed
	}

	-- metatable for class's env
	_MetaClsEnv = _MetaClsEnv or {}
	do
		_MetaClsEnv.__index = function(self, key)
			local info = _ClsEnv2Info[self]

			-- Check owner
			if key == info.Name then
				return info.Owner
			end

			-- Check keywords
			if _KeyWord4ClsEnv[key] then
				return _KeyWord4ClsEnv[key]
			end

			-- Check namespace
			if info.NameSpace then
				if key == _NSInfo[info.NameSpace].Name then
					self[key] = info.NameSpace
					return rawget(self, key)
				elseif info.NameSpace[key] then
					self[key] = info.NameSpace[key]
					return rawget(self, key)
				end
			end

			-- Check imports
			if info.Import4Env then
				for _, ns in ipairs(info.Import4Env) do
					if key == _NSInfo[ns].Name then
						self[key] = ns
						return rawget(self, key)
					elseif ns[key] then
						self[key] = ns[key]
						return rawget(self, key)
					end
				end
			end

			-- Check Base
			if info.BaseEnv then
				return info.BaseEnv[key]
			end
		end

		_MetaClsEnv.__newindex = function(self, key, value)
			local info = _ClsEnv2Info[self]

			if _KeyWord4ClsEnv[key] then
				error(("'%s' is a keyword."):format(key), 2)
			end

			if key == info.Name then
				if type(value) == "function" then
					rawset(info, "Constructor", value)
					return
				else
					error(("'%s' must be a function as constructor."):format(key), 2)
				end
			end

			if _KeyMeta[key] ~= nil then
				if type(value) == "function" then
					local rMeta = _KeyMeta[key] and key or "_"..key
					SetMetaFunc(rMeta, info.ChildClass, info.MetaTable[rMeta], value)
					info.MetaTable[rMeta] = value
					return
				else
					error(("'%s' must be a function."):format(key), 2)
				end
			end

			rawset(self, key, value)
		end
	end

	-- metatable for class cache
	_MetaClsCache4Script = _MetaClsCache4Script or {}
	do
		_BaseScripts = _BaseScripts or {
			OnScriptHandlerChanged = true,
		}

		_MetaClsCache4Script.__index = function(self, key)
			local info = _ClsCache2Info[self]

			if info.Script[key] ~= nil then
				rawset(self, key, info.Script[key])
				return rawget(self, key)
			end

			if info.SuperClass then
				if _NSInfo[info.SuperClass].Cache4Script[key] ~= nil then
					rawset(self, key, _NSInfo[info.SuperClass].Cache4Script[key])
					return rawget(self, key)
				end
			elseif _BaseScripts[key] ~= nil then
				rawset(self, key,_BaseScripts[key])
				return rawget(self, key)
			end
		end
	end

	_MetaClsCache4Prop = _MetaClsCache4Prop or {}
	do
		_MetaClsCache4Prop.__index = function(self, key)
			local info = _ClsCache2Info[self]

			if info.Property[key] then
				rawset(self, key, info.Property[key])
				return rawget(self, key)
			end

			if info.SuperClass and _NSInfo[info.SuperClass].Cache4Property[key] then
				rawset(self, key, _NSInfo[info.SuperClass].Cache4Property[key])
				return rawget(self, key)
			end
		end
	end

	_MetaClsCache4Method = _MetaClsCache4Method or {}
	do
		_MetaClsCache4Method.__index = function(self, key)
			local info = _ClsCache2Info[self]

			if type(rawget(info.ClassEnv, key)) == "function" then
				rawset(self, key, rawget(info.ClassEnv, key))
				return rawget(self, key)
			end

			if info.SuperClass and _NSInfo[info.SuperClass].Cache4Method[key] ~= nil then
				rawset(self, key, _NSInfo[info.SuperClass].Cache4Method[key])
				return rawget(self, key)
			end
		end
	end

	-- metatable for ScriptHandler
	_MetaScriptHandler = _MetaScriptHandler or {}
	do
		_MetaScriptHandler.__index = {
			Add = function(self, func)
				if type(func) ~= "function" then
					error("Usage: obj.OnXXXX:Add(func)", 2)
				end

				for _, f in ipairs(self) do
					if f == func then
						return self
					end
				end

				tinsert(self, func)

				if self._Owner and self._Name then
					CallScriptWithoutCreate(self._Owner, "OnScriptHandlerChanged", self._Name)
				end

				return self
			end,
			Remove = function(self, func)
				local flag = false

				if type(func) ~= "function" then
					error("Usage: obj.OnXXXX:Remove(func)", 2)
				end

				for i, f in ipairs(self) do
					if f == func then
						tremove(self, i)
						flag = true
						break
					end
				end

				if flag and self._Owner and self._Name then
					CallScriptWithoutCreate(self._Owner, "OnScriptHandlerChanged", self._Name)
				end

				return self
			end,
			Clear = function(self)
				local flag = false

				for i = #self, 0, -1 do
					if not flag and self[i] then
						flag = true
					end
					tremove(self, i)
				end

				if flag and self._Owner and self._Name then
					CallScriptWithoutCreate(self._Owner, "OnScriptHandlerChanged", self._Name)
				end

				return self
			end,
			IsEmpty = function(self)
				return #self == 0 and self[0] == nil
			end,
		}

		_MetaScriptHandler.__add = function(self, func)
			if type(func) ~= "function" then
				error("Usage: obj.OnXXXX = obj.OnXXXX + func", 2)
			end

			for _, f in ipairs(self) do
				if f == func then
					return self
				end
			end

			tinsert(self, func)

			if self._Owner and self._Name then
				CallScriptWithoutCreate(self._Owner, "OnScriptHandlerChanged", self._Name)
			end

			return self
		end

		_MetaScriptHandler.__sub = function(self, func)
			local flag = false

			if type(func) ~= "function" then
				error("Usage: obj.OnXXXX = obj.OnXXXX - func", 2)
			end

			for i, f in ipairs(self) do
				if f == func then
					tremove(self, i)
					flag = true
					break
				end
			end

			if flag and self._Owner and self._Name then
				CallScriptWithoutCreate(self._Owner, "OnScriptHandlerChanged", self._Name)
			end

			return self
		end

		local create = coroutine.create
		local resume = coroutine.resume
		local status = coroutine.status

		_MetaScriptHandler.__call = function(self, obj, ...)
			if not obj then
				error("Usage: obj:OnXXXX(...).", 2)
			end

			local chk = false
			local ret = false

			for i = 1, #self do
				if self._ThreadActived then
					local thread = create(self[i])
					chk, ret = resume(thread, obj, ...)
					if status(thread) ~= "dead" then
						-- not stop when the thread not dead
						ret = nil
					end
				else
					chk, ret = pcall(self[i], obj, ...)
				end
				if not chk then
					return errorhandler(ret)
				end

				if not rawget(obj, "__Scripts") then
					-- means it's disposed
					ret = true
				end
				if ret then
					break
				end
			end

			if not ret and self[0] then
				if self._ThreadActived then
					chk, ret = resume(create(self[0]), obj, ...)
				else
					chk, ret = pcall(self[0], obj, ...)
				end
				if not chk then
					return errorhandler(ret)
				end
			end
		end
	end

	_MetaScripts = _MetaScripts or {}
	do
		_MetaScripts.__index = function(self, key)
			-- Check Script
			local cls = self._Owner and getmetatable(self._Owner) and getmetatable(self._Owner).__class

			if _NSInfo[cls].Cache4Script[key] == nil then
				return
			end

			-- Add Script Handler
			rawset(self, key, setmetatable({_Owner = self._Owner, _Name = key}, _MetaScriptHandler))
			return rawget(self, key)
		end

		_MetaScripts.__newindex = function(self, key, value)
		end
	end

	function IsChildClass(cls, child)
		if not cls or not child or not _NSInfo[cls] or _NSInfo[cls].Type ~= TYPE_CLASS or not _NSInfo[child] or _NSInfo[child].Type ~= TYPE_CLASS then
			return false
		end

		if cls == child then
			return true
		end

		local info = _NSInfo[child]

		while info and info.SuperClass and info.SuperClass ~= cls do
			info = _NSInfo[info.SuperClass]
		end

		if info and info.SuperClass == cls then
			return true
		end

		return false
	end

	function SetMetaFunc(meta, sub, pre, now)
		if sub and pre ~= now then
			for cls in pairs(sub) do
				local info = _NSInfo[cls]

				if info.MetaTable[meta] == pre then
					info.MetaTable[meta] = now

					SetMetaFunc(meta, info.ChildClass, pre, now)
				end
			end
		end
	end

	function ClearCache(cls)
		local info = _NSInfo[cls]

		wipe(info.Cache4Script)
		wipe(info.Cache4Property)
		wipe(info.Cache4Method)

		-- Clear child's cache
		if info.ChildClass then
			for subcls in pairs(info.ChildClass) do
				ClearCache(subcls)
			end
		end
	end

	function CheckProperty(prop, value)
		if prop and prop.Type then
			local ok, ret = pcall(prop.Type.Validate, prop.Type, value)

			if not ok then
				ret = strtrim(ret:match(":%d+:(.*)$") or ret)

				if ret:find("%%s") then
					ret = ret:gsub("%%s[_%w]*", prop.Name)
				end

				error(ret, 3)
			end

			return ret
		end

		return value
	end

	function CallScriptWithoutCreate(self, scriptName, ...)
		if rawget(self, "__Scripts") and rawget(self.__Scripts, scriptName) then
			return self[scriptName](self, ...)
		end
	end

	function Class2Obj(cls, ...)
		local ret, ok, obj

		local info = _NSInfo[cls]
		local cache = info.Cache4Method

		-- Check if this class has __exist so no need to create again.
		if type(info.MetaTable.__exist) == "function" then
			ret = {pcall(info.MetaTable.__exist, cls, ...)}

			if not ret[1] then
				ret[2] = strtrim(ret[2]:match(":%d+:(.*)$") or ret[2])

				error(ret[2], 3)
			end

			if type(ret[2]) == "table" then
				obj = ret[2]
				local meta = getmetatable(obj)

				if meta and meta.__class then
					if meta ~= info.MetaTable then
						setmetatable(obj, info.MetaTable)
					end
					return obj
				end
			elseif ret[2] == false then
				-- if the __exist modify the args.

				-- Create new object
				local constructor = info.Constructor
				if type(constructor) ~= "function" then
					error(("no Constructor found for class %s."):format(tostring(cls)), 3)
				end

				ok, obj = pcall(constructor, unpack(ret, 3))

				if not ok then
					obj = strtrim(obj:match(":%d+:(.*)$") or obj)

					error(obj, 3)
				end

				if type(obj) == "table" then
					setmetatable(obj, info.MetaTable)
				end

				return obj
			end
		end

		-- Create new object
		local constructor = info.Constructor
		if type(constructor) ~= "function" then
			error(("no Constructor found for class %s."):format(tostring(cls)), 3)
		end

		ok, obj = pcall(constructor, ...)

		if not ok then
			obj = strtrim(obj:match(":%d+:(.*)$") or obj)

			error(obj, 3)
		end

		if type(obj) == "table" then
			setmetatable(obj, info.MetaTable)
		end

		return obj
	end

	------------------------------------
	--- Create class in currect environment's namespace or default namespace
	-- @name class
	-- @class function
	-- @param name the class's name
	-- @usage class "Form"
	------------------------------------
	function class(name)
		if type(name) ~= "string" or name:find("%.") or not name:match("[_%w]+") then
			error([[Usage: class "classname"]], 2)
		end
		local fenv = getfenv(2)
		local ns = GetNameSpace4Env(fenv)

		-- Create class or get it
		local cls

		name = name:match("[_%w]+")

		if ns then
			cls = BuildNameSpace(ns, name)

			if _NSInfo[cls] then
				if _NSInfo[cls].Type and _NSInfo[cls].Type ~= TYPE_CLASS then
					error(("%s is existed as %s, not class."):format(name, tostring(_NSInfo[cls].Type)), 2)
				end

				if _NSInfo[cls].BaseEnv and _NSInfo[cls].BaseEnv ~= fenv then
					error(("%s is defined in another place, can't be defined here."):format(name), 2)
				end
			end
		else
			cls = fenv[name]

			if not(_NSInfo[cls] and _NSInfo[cls].BaseEnv == fenv and _NSInfo[cls].NameSpace == nil and _NSInfo[cls].Type == TYPE_CLASS) then
				cls = BuildNameSpace(nil, name)
			end
		end

		if not cls then
			error("no class is created.", 2)
		end

		-- save class to the environment
		rawset(fenv, name, cls)

		-- Build class
		info = _NSInfo[cls]
		info.Type = TYPE_CLASS
		info.NameSpace = ns
		info.BaseEnv = fenv
		info.Script = info.Script or {}
		info.Property = info.Property or {}

		info.ClassEnv = info.ClassEnv or setmetatable({}, _MetaClsEnv)
		_ClsEnv2Info[info.ClassEnv] = info

		-- Clear
		info.Constructor = nil
		wipe(info.Property)
		wipe(info.Script)
		for i, v in pairs(info.ClassEnv) do
			if type(v) == "function" then
				info.ClassEnv[i] = nil
			end
		end

		-- Set namespace
		SetNameSpace4Env(info.ClassEnv, cls)

		-- Cache
		info.Cache4Script = info.Cache4Script or setmetatable({}, _MetaClsCache4Script)
		info.Cache4Property = info.Cache4Property or setmetatable({}, _MetaClsCache4Prop)
		info.Cache4Method = info.Cache4Method or setmetatable({}, _MetaClsCache4Method)
		_ClsCache2Info[info.Cache4Script] = info
		_ClsCache2Info[info.Cache4Property] = info
		_ClsCache2Info[info.Cache4Method] = info

		ClearCache(cls)

		-- SuperClass
		local prevInfo = info.SuperClass and _NSInfo[info.SuperClass]

		if prevInfo and prevInfo.ChildClass then
			prevInfo.ChildClass[info.Owner] = nil
		end

		info.SuperClass = nil

		-- Import
		info.Import4Env = info.Import4Env or {}
		wipe(info.Import4Env)

		-- MetaTable
		info.MetaTable = info.MetaTable or {}
		do
			local MetaTable = info.MetaTable
			local rMeta

			-- Clear
			for meta, flag in pairs(_KeyMeta) do
				rMeta = flag and meta or "_"..meta
				SetMetaFunc(rMeta, info.ChildClass, MetaTable[rMeta], nil)
				MetaTable[rMeta] = nil
			end

			local ClassEnv = info.ClassEnv
			local Cache4Script = info.Cache4Script
			local Cache4Property = info.Cache4Property
			local Cache4Method = info.Cache4Method
			local ClassName = info.Name

			MetaTable.__class = cls

			MetaTable.__index = function(self, key)
				-- Property Get
				if Cache4Property[key] then
					if Cache4Property[key]["Get"] then
						return Cache4Property[key]["Get"](self)
					else
						error(("%s is write-only."):format(tostring(key)),2)
					end
				end

				-- Frame Func
				if type(key) == "string" and (not key:find("^_")) and Cache4Method and Cache4Method[key] then
					return Cache4Method[key]
				end

				-- Scripts
				if Cache4Script[key] ~= nil then
					if type(rawget(self, "__Scripts")) ~= "table" or getmetatable(self.__Scripts) ~= _MetaScripts then
						rawset(self, "__Scripts", setmetatable({_Owner = self}, _MetaScripts))
					end

					return self.__Scripts[key]
				end

				-- Custom index metametods
				if rawget(MetaTable, "___index") then
					if type(rawget(MetaTable, "___index")) == "table" then
						return rawget(MetaTable, "___index")[key]
					elseif type(rawget(MetaTable, "___index")) == "function" then
						local ok, ret = pcall(rawget(MetaTable, "___index"), self, key)

						if not ok then
							ret = strtrim(ret:match(":%d+:(.*)$") or ret)

							error(ret, 2)
						end

						return ret
					end
				end
			end

			MetaTable.__newindex = function(self, key, value)
				-- Property Set
				if Cache4Property[key] then
					if Cache4Property[key]["Set"] then
						return Cache4Property[key]["Set"](self, CheckProperty(Cache4Property[key], value))
					else
						error(("%s is read-only."):format(tostring(key)),2)
					end
				end

				-- Scripts
				if Cache4Script[key] ~= nil then
					if type(rawget(self, "__Scripts")) ~= "table" or getmetatable(self.__Scripts) ~= _MetaScripts then
						rawset(self, "__Scripts", setmetatable({_Owner = self}, _MetaScripts))
					end

					if value == nil or type(value) == "function" then
						if Cache4Script[key] then
							if self.__Scripts[key][0] ~= value then
								rawset(self.__Scripts[key], 0, value)
								CallScriptWithoutCreate(self, "OnScriptHandlerChanged", key)
							end
						else
							error(("%s is not supported for class '%s'."):format(tostring(key), ClassName), 2)
						end
					elseif type(value) == "table" and getmetatable(value) == _MetaScriptHandler then
						if value == self.__Scripts[key] then
							return
						end

						for i = #self.__Scripts[key], 0, -1 do
							tremove(self.__Scripts[key], i)
						end

						for i =0, #value do
							self.__Scripts[key][i] = value[i]
						end

						CallScriptWithoutCreate(self, "OnScriptHandlerChanged", key)
					else
						error("can't set this value to a scipt handler.", 2)
					end

					return
				end

				-- Custom newindex metametods
				if type(rawget(MetaTable, "___newindex")) == "function" then
					local ok, ret = pcall(rawget(MetaTable, "___newindex"), self, key, value)

					if not ok then
						ret = strtrim(ret:match(":%d+:(.*)$") or ret)

						error(ret, 2)
					end

					return ret
				end

				rawset(self,key,value)			-- Other key can be set as usual
			end
		end

		-- Set the environment to class's environment
		setfenv(2, info.ClassEnv)
	end

	------------------------------------
	--- Set the current class' super class
	-- @name inherit
	-- @class function
	-- @param name the namespace's name list, using "." to split.
	-- @usage inherit "System.Widget.Frame"
	------------------------------------
	function inherit(name)
		if name and type(name) ~= "string" and not IsNameSpace(name) then
			error([[Usage: inherit "namespace.classname"]], 2)
		end

		if type(name) == "string" and name:find("%.%s*%.") then
			error("the namespace 's name can't have empty string between dots.", 2)
		end

		local env = getfenv(2)

		if rawget(_ClsEnv2Info, env) == nil then
			error("can't using inherit here.", 2)
		end

		local info = _ClsEnv2Info[env]

		local prevInfo = info.SuperClass and _NSInfo[info.SuperClass]

		if prevInfo then
			error("this class already have a super class.", 2)
		end

		local superCls

		if type(name) == "string" then
			superCls = GetNameSpace(info.NameSpace, name)

			if not superCls then
				for subname in name:gmatch("[^%.]+") do
					subname = subname:match("[_%w]+")

					if not subname or subname == "" then
						error("the namespace's name must be composed with number, string or '_'.", 2)
					end

					if not superCls then
						superCls = info.BaseEnv[subname]
					else
						superCls = superCls[subname]
					end

					if not IsNameSpace(superCls) then
						error(("no class is found with the name : %s"):format(name), 2)
					end
				end
			end
		else
			superCls = name
		end

		local superInfo = _NSInfo[superCls]

		if not superInfo or superInfo.Type ~= TYPE_CLASS then
			error("Usage: inherit (class) : 'class' - class expected", 2)
		end

		if IsChildClass(info.Owner, superCls) then
			error(("%s is inherited from %s, can't be used as super class."):format(tostring(superCls), tostring(info.Owner)), 2)
		end

		superInfo.ChildClass = superInfo.ChildClass or {}
		superInfo.ChildClass[info.Owner] = true
		info.SuperClass = superCls

		-- Copy Metatable
		local rMeta
		for meta, flag in pairs(_KeyMeta) do
			rMeta = flag and meta or "_"..meta

			if info.MetaTable[rMeta] == nil and superInfo.MetaTable[rMeta] then
				SetMetaFunc(rMeta, info.ChildClass, nil, superInfo.MetaTable[rMeta])
				info.MetaTable[rMeta] = superInfo.MetaTable[rMeta]
			end
		end
	end

	------------------------------------
	--- import classes from the given name's namespace to the current environment
	-- @name import
	-- @class function
	-- @param name the namespace's name list, using "." to split.
	-- @usage import "System.Widget"
	------------------------------------
	function import(name)
		if type(name) ~= "string" and not IsNameSpace(name) then
			error([[Usage: import "namespaceA.namespaceB"]], 2)
		end

		if type(name) == "string" and name:find("%.%s*%.") then
			error("the namespace 's name can't have empty string between dots.", 2)
		end

		local env = getfenv(2)

		local info = _ClsEnv2Info[env]

		if not info then
			error("can't use import here.", 2)
		end

		local ns

		if type(name) == "string" then
			ns = GetNameSpace(GetDefaultNameSpace(), name)
		elseif IsNameSpace(name) then
			ns = name
		end

		if not ns then
			error(("no namespace is found with name : %s"):format(name), 2)
		end

		info.Import4Env = info.Import4Env or {}

		for _, v in ipairs(info.Import4Env) do
			if v == ns then
				return
			end
		end

		tinsert(info.Import4Env, ns)
	end

	------------------------------------
	--- Add or remove a script for current class
	-- @name script
	-- @class function
	-- @param name the name of the script, if started with "-" means to remove this script
	-- @usage script "OnClick"
	-- @usage script "-OnClick"
	------------------------------------
	function script(name)
		if type(name) ~= "string" or name:find("^_") then
			error([[Usage: script "[-]scriptName"]], 2)
		end

		local env = getfenv(2)

		local info = _ClsEnv2Info[env]

		if not info then
			error("can't use script here.", 2)
		end

		local flag

		name, flag = name:gsub("^-", "")

		info.Script[name] = (flag == 0)
	end

	local function SetProperty2Cls(info, name, set)
		if type(set) ~= "table" then
			error([=[Usage: property "propertyName" {
				Get = function(self)
					-- return the property's value
				end,
				Set = function(self, value)
					-- Set the property's value
				end,
				Type = Type1 [+ Type2 [+ nil]],	-- set the property's type
			}]=], 2)
		end

		for i, v in pairs(set) do
			if type(i) == "string" then
				if i:lower() == "get" then
					set.Get = v
				elseif i:lower() == "set" then
					set.Set = v
				elseif i:lower() == "type" then
					set.Type = v
				end
			end
		end

		local prop = info.Property[name] or {}
		info.Property[name] = prop

		wipe(prop)

		prop.Name = name
		prop.Get = type(set.Get) == "function" and set.Get
		prop.Set = type(set.Set) == "function" and set.Set

		if set.Type then
			local ok, _type = pcall(BuildType, set.Type, name)
			if ok then
				prop.Type = _type
			else
				_type = strtrim(_type:match(":%d+:(.*)$") or _type)

				error(_type, 3)
			end
		end
	end

	------------------------------------
	--- set a propert to the current class
	-- @name property
	-- @class function
	-- @param name the name of the property
	-- @usage property "Title" {
	--		Get = function(self)
	--			-- return the property's value
	--		end,
	--		Set = function(self, value)
	--			-- Set the property's value
	--		end,
	--		Type = "XXXX",	-- set the property's type
	--}
	------------------------------------
	function property(name)
		if type(name) ~= "string" or strtrim(name:match("[_%w]+")) == "" then
			error([=[Usage: property "propertyName" {
				Get = function(self)
					-- return the property's value
				end,
				Set = function(self, value)
					-- Set the property's value
				end,
				Type = Type1 [+ Type2 [+ nil]],	-- set the property's type
			}]=], 2)
		end

		name = name:match("[_%w]+")

		local env = getfenv(2)

		local info = _ClsEnv2Info[env]

		if not info then
			error("can't use property here.", 2)
		end

		return function(set)
			return SetProperty2Cls(info, name, set)
		end
	end

	------------------------------------
	--- End the class's definition and restore the environment
	-- @name class
	-- @class function
	-- @param name the name of the class
	-- @usage endclass "Form"
	------------------------------------
	function endclass(name)
		if type(name) ~= "string" or name:find("%.") then
			error([[Usage: endclass "classname"]], 2)
		end

		local env = getfenv(2)

		while rawget(_ClsEnv2Info, env) do
			local info = _ClsEnv2Info[env]

			if info.Name == name then
				setfenv(2, info.BaseEnv)
				return
			end

			env = info.BaseEnv
		end

		error(("no class is found with name: %s"):format(name), 2)
	end

	_KeyWord4ClsEnv.class = class
	_KeyWord4ClsEnv.inherit = inherit
	_KeyWord4ClsEnv.import = import
	_KeyWord4ClsEnv.script = script
	_KeyWord4ClsEnv.property = property
	_KeyWord4ClsEnv.endclass = endclass
end

------------------------------------------------------
-- Enum
------------------------------------------------------
do
	local function BuildEnum(info, set)
		if type(set) ~= "table" then
			error([[Usage: enum "enumName" {
				"enumValue1",
				"enumValue2",
			}]], 2)
		end

		info.Enum = info.Enum or {}

		wipe(info.Enum)

		for i, v in pairs(set) do
			if type(i) == "string" then
				info.Enum[i:upper()] = v
			elseif type(v) == "string" then
				info.Enum[v:upper()] = v
			end
		end
	end

	------------------------------------
	--- create a enumeration
	-- @name enum
	-- @class function
	-- @param name the name of the enum
	-- @usage enum "ButtonState" {
	--		"PUSHED",
	--		"NORMAL",
	--}
	------------------------------------
	function enum(name)
		if type(name) ~= "string" or name:find("%.") or strtrim(name:match("[_%w]+") or "") == "" then
			error([[Usage: enum "enumName" {
				"enumValue1",
				"enumValue2",
			}]], 2)
		end

		name = name:match("[_%w]+")

		local fenv = getfenv(2)
		local ns = GetNameSpace4Env(fenv)

		-- Create class or get it
		local enm

		if ns then
			enm = BuildNameSpace(ns, name)

			if _NSInfo[cls] then
				if _NSInfo[cls].Type and _NSInfo[cls].Type ~= TYPE_ENUM then
					error(("%s is existed as %s, not enumeration."):format(name, tostring(_NSInfo[cls].Type)), 2)
				end
			end
		else
			enm = fenv[name]

			if not(_NSInfo[enm] and _NSInfo[enm].Type == TYPE_ENUM) then
				enm = BuildNameSpace(nil, name)
			end
		end

		if not enm then
			error("no enumeration is created.", 2)
		end

		-- save class to the environment
		rawset(fenv, name, enm)

		-- Build enm
		local info = _NSInfo[enm]
		info.Type = TYPE_ENUM
		info.NameSpace = ns

		return function(set)
			return BuildEnum(info, set)
		end
	end
end

------------------------------------------------------
-- Struct
------------------------------------------------------
do
	_StructEnv2Info = _StructEnv2Info or setmetatable({}, {__mode = "kv",})

	_KeyWord4StrtEnv = _KeyWord4StrtEnv or {}

	-- metatable for class's env
	_MetaStrtEnv = _MetaStrtEnv or {}
	do
		_MetaStrtEnv.__index = function(self, key)
			local info = _StructEnv2Info[self]

			-- Check owner
			if key == info.Name then
				return info.Owner
			end

			if key == "Validate" then
				return info.UserValidate
			end

			-- Check keywords
			if _KeyWord4StrtEnv[key] then
				return _KeyWord4StrtEnv[key]
			end

			-- Check namespace
			if info.NameSpace then
				if key == _NSInfo[info.NameSpace].Name then
					rawset(self, key, info.NameSpace)
					return rawget(self, key)
				elseif info.NameSpace[key] then
					rawset(self, key, info.NameSpace[key])
					return rawget(self, key)
				end
			end

			-- Check imports
			if info.Import4Env then
				for _, ns in ipairs(info.Import4Env) do
					if key == _NSInfo[ns].Name then
						self[key] = ns
						return rawget(self, key)
					elseif ns[key] then
						self[key] = ns[key]
						return rawget(self, key)
					end
				end
			end

			-- Check Base
			if info.BaseEnv then
				return info.BaseEnv[key]
			end
		end

		_MetaStrtEnv.__newindex = function(self, key, value)
			local info = _StructEnv2Info[self]

			if _KeyWord4StrtEnv[key] then
				error(("the '%s' is a keyword."):format(key), 2)
			end

			if key == info.Name then
				-- error(("the '%s' is the struct name, can't be used."):format(key), 2)
				if type(value) == "function" then
					rawset(info, "Constructor", value)
					return
				else
					error(("the '%s' must be a function as constructor."):format(key), 2)
				end
			end

			if key == "Validate" then
				if value == nil or type(value) == "function" then
					info.UserValidate = value
					return
				else
					error(("the '%s' must be a function used for validation."):format(key), 2)
				end
			end

			if type(key) == "string" and (value == nil or IsType(value) or IsNameSpace(value)) then
				local ok, ret = pcall(BuildType, value, key)

				if ok then
					rawset(self, key, ret)

					info.Members = info.Members or {}
					tinsert(info.Members, key)

					return
				else
					ret = strtrim(ret:match(":%d+:(.*)$") or ret)
					error(ret, 2)
				end
			end

			rawset(self, key, value)
		end
	end

	function ValidateStruct(strt, value)
		local info = _NSInfo[strt]

		if info.Members then
			assert(type(value) == "table", ("%s must be a table, got %s."):format("%s", type(value)))

			for _, n in ipairs(info.Members) do
				value[n] = info.StructEnv[n]:Validate(value[n])
			end
		end

		if type(info.UserValidate) == "function" then
			value = info.UserValidate(value)
		end

		return value
	end

	function Struct2Obj(strt, ...)
		local info = _NSInfo[strt]

		if type(info.Constructor) == "function" then
			local ok, ret = pcall(info.Constructor, ...)
			if ok then
				return ret
			else
				ret = strtrim(ret:match(":%d+:(.*)$") or ret)

				error(ret, 3)
			end
		elseif info.Members and #info.Members > 0 then
			local ret = {}

			for i, n in ipairs(info.Members) do
				ret[n] = select(i, ...)
			end

			local ok, value = pcall(ValidateStruct, strt, ret)

			if ok then
				return value
			else
				value = strtrim(value:match(":%d+:(.*)$") or value)
				value = value:gsub("%%s%.", ""):gsub("%%s", "")

				local args = ""
				for i, n in ipairs(info.Members) do
					if info.StructEnv[n]:Is(nil) and not args:find("%[") then
						n = "["..n
					end
					if i == 1 then
						args = n
					else
						args = args..", "..n
					end
				end
				if args:find("%[") then
					args = args.."]"
				end
				error(("Usage : %s(%s) - %s"):format(tostring(strt), args, value), 3)
			end
		else
			error(("struct '%s' is abstract."):format(tostring(strt)), 3)
		end
	end

	------------------------------------
	--- create a structure
	-- @name struct
	-- @class function
	-- @param name the name of the enum
	-- @usage struct "ButtonState" {
	--
	--}
	------------------------------------
	function struct(name)
		if type(name) ~= "string" or name:find("%.") or not name:match("[_%w]+") then
			error([[Usage: struct "structname"]], 2)
		end
		local fenv = getfenv(2)
		local ns = GetNameSpace4Env(fenv)

		-- Create class or get it
		local strt

		name = name:match("[_%w]+")

		if ns then
			strt = BuildNameSpace(ns, name)

			if _NSInfo[strt] then
				if _NSInfo[strt].Type and _NSInfo[strt].Type ~= TYPE_STRUCT then
					error(("%s is existed as %s, not struct."):format(name, tostring(_NSInfo[strt].Type)), 2)
				end

				if _NSInfo[strt].BaseEnv and _NSInfo[strt].BaseEnv ~= fenv then
					error(("%s is defined in another place, can't be defined here."):format(name), 2)
				end
			end
		else
			strt = fenv[name]

			if not(_NSInfo[strt] and _NSInfo[strt].BaseEnv == fenv and _NSInfo[strt].NameSpace == nil and _NSInfo[strt].Type == TYPE_STRUCT) then
				strt = BuildNameSpace(nil, name)
			end
		end

		if not strt then
			error("no struct is created.", 2)
		end

		-- save class to the environment
		rawset(fenv, name, strt)

		-- Build class
		info = _NSInfo[strt]
		info.Type = TYPE_STRUCT
		info.NameSpace = ns
		info.BaseEnv = fenv
		info.Members = nil
		info.UserValidate = nil

		info.StructEnv = info.StructEnv or setmetatable({}, _MetaStrtEnv)
		_StructEnv2Info[info.StructEnv] = info

		-- Clear
		info.Constructor = nil

		wipe(info.StructEnv)

		-- Set namespace
		SetNameSpace4Env(info.StructEnv, strt)

		-- Set the environment to class's environment
		setfenv(2, info.StructEnv)
	end

	------------------------------------
	--- import classes from the given name's namespace to the current environment
	-- @name import
	-- @class function
	-- @param name the namespace's name list, using "." to split.
	-- @usage import "System.Widget"
	------------------------------------
	function import(name)
		if type(name) ~= "string" and not IsNameSpace(name) then
			error([[Usage: import "namespaceA.namespaceB"]], 2)
		end

		if type(name) == "string" and name:find("%.%s*%.") then
			error("the namespace 's name can't have empty string between dots.", 2)
		end

		local env = getfenv(2)

		local info = _StructEnv2Info[env]

		if not info then
			error("can't use import here.", 2)
		end

		local ns

		if type(name) == "string" then
			ns = GetNameSpace(GetDefaultNameSpace(), name)
		elseif IsNameSpace(name) then
			ns = name
		end

		if not ns then
			error(("no namespace is found with name : %s"):format(name), 2)
		end

		info.Import4Env = info.Import4Env or {}

		for _, v in ipairs(info.Import4Env) do
			if v == ns then
				return
			end
		end

		tinsert(info.Import4Env, ns)
	end

	------------------------------------
	--- End the class's definition and restore the environment
	-- @name class
	-- @class function
	-- @param name the name of the class
	-- @usage endclass "Form"
	------------------------------------
	function endstruct(name)
		if type(name) ~= "string" or name:find("%.") then
			error([[Usage: endstruct "structname"]], 2)
		end

		local env = getfenv(2)

		while rawget(_StructEnv2Info, env) do
			local info = _StructEnv2Info[env]

			if info.Name == name then
				setfenv(2, info.BaseEnv)
				return
			end

			env = info.BaseEnv
		end

		error(("no struct is found with name: %s"):format(name), 2)
	end

	_KeyWord4StrtEnv.struct = struct
	_KeyWord4StrtEnv.import = import
	_KeyWord4StrtEnv.endstruct = endstruct
end

------------------------------------------------------
-- System.Reflector
------------------------------------------------------
do
	namespace "System"

	class "Reflector"
		_NSInfo = _NSInfo
		TYPE_CLASS = TYPE_CLASS
		TYPE_STRUCT = TYPE_STRUCT
		TYPE_ENUM = TYPE_ENUM

		local GetNameSpace = GetNameSpace
		local GetDefaultNameSpace = GetDefaultNameSpace
		local IsChildClass = IsChildClass
		local unpack = unpack
		local tinsert = tinsert
		local sort = table.sort
		local IsType = IsType

		------------------------------------
		--- Get the namespace for the name
		-- @name ForName
		-- @class function
		-- @param name the namespace's name, split by "."
		-- @return namespace the namespace
		-- @usage System.Reflector.ForName("System")
		------------------------------------
		function ForName(name)
			return GetNameSpace(GetDefaultNameSpace(), name)
		end

		------------------------------------
		--- Get the type for the namespace
		-- @name GetType
		-- @class function
		-- @param name the namespace
		-- @return type
		-- @usage System.Reflector.GetType("System.Object")
		------------------------------------
		function GetType(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			return ns and rawget(_NSInfo, ns) and _NSInfo[ns].Type
		end

		------------------------------------
		--- Get the name for the namespace
		-- @name GetName
		-- @class function
		-- @param name the namespace
		-- @return name
		-- @usage System.Reflector.GetName(System.Object)
		------------------------------------
		function GetName(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			return ns and rawget(_NSInfo, ns) and _NSInfo[ns].Name
		end

		------------------------------------
		--- Get the full name for the namespace
		-- @name GetFullName
		-- @class function
		-- @param name the namespace
		-- @return name
		-- @usage System.Reflector.GetFullName(System.Object)
		------------------------------------
		function GetFullName(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			return GetFullName4NS(ns)
		end

		------------------------------------
		--- Get the superclass for the class
		-- @name GetSuperClass
		-- @class function
		-- @param class
		-- @return superclass
		-- @usage System.Reflector.GetSuperClass(System.Object)
		------------------------------------
		function GetSuperClass(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			return ns and rawget(_NSInfo, ns) and _NSInfo[ns].SuperClass
		end

		------------------------------------
		--- Check if the object is a NameSpace
		-- @name IsNameSpace
		-- @class function
		-- @param obj
		-- @return boolean
		-- @usage System.Reflector.IsNameSpace(System.Object)
		------------------------------------
		function IsNameSpace(ns)
			if type(ns) == "string" then ns = ForName(ns) end
			
			return ns and rawget(_NSInfo, ns) and true or false
		end
		
		------------------------------------
		--- Check if the namespace is a class
		-- @name IsClass
		-- @class function
		-- @param namespace
		-- @return boolean
		-- @usage System.Reflector.IsClass(System.Object)
		------------------------------------
		function IsClass(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			return ns and rawget(_NSInfo, ns) and _NSInfo[ns].Type == TYPE_CLASS or false
		end

		------------------------------------
		--- Check if the namespace is an abstract class
		-- @name IsAbstractClass
		-- @class function
		-- @param namespace
		-- @return boolean
		-- @usage System.Reflector.IsAbstractClass(System.Object)
		------------------------------------
		function IsAbstractClass(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			return not(ns and rawget(_NSInfo, ns) and _NSInfo[ns].Type == TYPE_CLASS and type(_NSInfo[ns].Constructor) == "function" or false)
		end

		------------------------------------
		--- Check if the namespace is a struct
		-- @name IsStruct
		-- @class function
		-- @param namespace
		-- @return boolean
		-- @usage System.Reflector.IsStruct(System.Object)
		------------------------------------
		function IsStruct(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			return ns and rawget(_NSInfo, ns) and _NSInfo[ns].Type == TYPE_STRUCT or false
		end

		------------------------------------
		--- Check if the namespace is an enum
		-- @name IsEnum
		-- @class function
		-- @param namespace
		-- @return boolean
		-- @usage System.Reflector.IsEnum(System.Object)
		------------------------------------
		function IsEnum(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			return ns and rawget(_NSInfo, ns) and _NSInfo[ns].Type == TYPE_ENUM or false
		end

		------------------------------------
		--- Get the sub namespace of the namespace
		-- @name GetSubNamespace
		-- @class function
		-- @param namespace
		-- @return table
		-- @usage System.Reflector.GetSubNamespace(System)
		------------------------------------
		function GetSubNamespace(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.SubNS then
				local ret = {}

				for key in pairs(info.SubNS) do
					tinsert(ret, key)
				end

				sort(ret)

				return ret
			end
		end

		------------------------------------
		--- Get the scripts of the class
		-- @name GetScripts
		-- @class function
		-- @param namespace
		-- @param noSuper
		-- @return table
		-- @usage System.Reflector.GetScripts(System.Object)
		------------------------------------
		function GetScripts(ns, noSuper)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.Type == TYPE_CLASS then
				local tmp = {}

				while info do
					for i, v in pairs(info.Script) do
						if tmp[i] == nil then
							tmp[i] = v
						end
					end

					if noSuper then break end
					
					info = info.SuperClass and _NSInfo[info.SuperClass]
				end

				local ret = {}

				for i, v in pairs(tmp) do
					if v then
						tinsert(ret, i)
					end
				end

				sort(ret)

				return ret
			end
		end

		------------------------------------
		--- Get the properties of the class
		-- @name GetProperties
		-- @class function
		-- @param namespace
		-- @param [noSuper] true if only get the properties defined in the class.
		-- @return table
		-- @usage System.Reflector.GetProperties(System.Object)
		------------------------------------
		function GetProperties(ns, noSuper)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.Type == TYPE_CLASS then
				local tmp = {}

				while info do
					for i, v in pairs(info.Property) do
						tmp[i] = true
					end

					if noSuper then	break end
					
					info = info.SuperClass and _NSInfo[info.SuperClass]
				end

				local ret = {}

				for i, v in pairs(tmp) do
					if v then
						tinsert(ret, i)
					end
				end

				sort(ret)

				return ret
			end
		end

		------------------------------------
		--- Get the methods of the class
		-- @name GetMethods
		-- @class function
		-- @param namespace
		-- @param noSuper
		-- @return table
		-- @usage System.Reflector.GetMethods(System.Object)
		------------------------------------
		function GetMethods(ns, noSuper)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.Type == TYPE_CLASS then
				local tmp = {}

				while info do
					for i, v in pairs(info.ClassEnv) do
						if type(v) == "function" then
							tmp[i] = true
						end
					end

					if noSuper then break end
					
					info = info.SuperClass and _NSInfo[info.SuperClass]
				end

				local ret = {}

				for i, v in pairs(tmp) do
					if v then
						tinsert(ret, i)
					end
				end

				sort(ret)

				return ret
			end
		end

		------------------------------------
		--- Get the property type of the class
		-- @name GetPropertyType
		-- @class function
		-- @param namespace
		-- @param propName
		-- @return table
		-- @usage System.Reflector.GetPropertyType(System.Object, "Name")
		------------------------------------
		function GetPropertyType(ns, propName)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.Type == TYPE_CLASS and info.Cache4Property[propName] then
				local ty = info.Cache4Property[propName].Type

				return ty and ty:Copy()
			end
		end

		------------------------------------
		--- whether the property is existed
		-- @name HasProperty
		-- @class function
		-- @param namespace
		-- @param propName
		-- @return boolean
		-- @usage System.Reflector.HasProperty(System.Object, "Name")
		------------------------------------
		function HasProperty(ns, propName)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.Type == TYPE_CLASS and info.Cache4Property[propName] then
				return true
			end

			return false
		end

		------------------------------------
		--- whether the property is readable
		-- @name IsPropertyReadable
		-- @class function
		-- @param namespace
		-- @param propName
		-- @return boolean
		-- @usage System.Reflector.IsPropertyReadable(System.Object, "Name")
		------------------------------------
		function IsPropertyReadable(ns, propName)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.Type == TYPE_CLASS and info.Cache4Property[propName] then
				return type(info.Cache4Property[propName].Get) == "function"
			end
		end

		------------------------------------
		--- whether the property is writable
		-- @name IsPropertyWritable
		-- @class function
		-- @param namespace
		-- @param propName
		-- @return boolean
		-- @usage System.Reflector.IsPropertyWritable(System.Object, "Name")
		------------------------------------
		function IsPropertyWritable(ns, propName)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.Type == TYPE_CLASS and info.Cache4Property[propName] then
				return type(info.Cache4Property[propName].Set) == "function"
			end
		end

		------------------------------------
		--- Get the enums of the enum
		-- @name GetEnums
		-- @class function
		-- @param namespace
		-- @return table
		-- @usage System.Reflector.GetEnums(System.SampleEnum)
		------------------------------------
		function GetEnums(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.Type == TYPE_ENUM then
				local tmp = {}

				for i in pairs(info.Enum) do
					tinsert(tmp, i)
				end

				sort(tmp)

				return tmp
			end
		end

		------------------------------------
		--- Get the enum index of the enum value
		-- @name ParseEnum
		-- @class function
		-- @param namespace
		-- @param value
		-- @return table
		-- @usage System.Reflector.ParseEnum(System.SampleEnum, 1)
		------------------------------------
		function ParseEnum(ns, value)
			if type(ns) == "string" then ns = ForName(ns) end

			if ns and _NSInfo[ns] and _NSInfo[ns].Type == TYPE_ENUM and _NSInfo[ns].Enum then
				for n, v in pairs(_NSInfo[ns].Enum) do
					if v == value then
						return n
					end
				end
			end
		end

		------------------------------------
		--- Check if the class has that script
		-- @name System.Reflector.HasScript
		-- @class function
		-- @param class the class that need to check
		-- @param script the script name
		-- @return true if the class has this script, otherwise false, nil when no class is found.
		-- @usage System.Reflector.HasScript(Addon, "OnEvent")
		------------------------------------
		function HasScript(cls, sc)
			if type(cls) == "string" then cls = ForName(cls) end

			local info = _NSInfo[cls]

			if info and info.Type == TYPE_CLASS then
				return info.Cache4Script[sc] or false
			end
		end

		------------------------------------
		--- Get the parts of the struct
		-- @name System.Reflector.GetStructParts
		-- @class function
		-- @param struct the struct
		-- @return a list to contains the parts' names
		-- @usage System.Reflector.GetStructParts(Position)
		------------------------------------
		function GetStructParts(ns)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and rawget(_NSInfo, ns)

			if info and info.Type == TYPE_STRUCT and info.Members and #info.Members > 0 then
				local tmp = {}

				for _, part in ipairs(info.Members) do
					tinsert(tmp, part)
				end

				return tmp
			end
		end

		------------------------------------
		--- Get the part's type of the struct
		-- @name System.Reflector.GetStructPart
		-- @class function
		-- @param struct the struct
		-- @param part the name of the part
		-- @return type of the part of the struct
		-- @usage System.Reflector.GetStructPart(Position, "x")
		------------------------------------
		function GetStructPart(ns, part)
			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and rawget(_NSInfo, ns)

			if info and info.Type == TYPE_STRUCT and info.Members and #info.Members > 0  then
				for _, p in ipairs(info.Members) do
					if p == part and IsType(info.StructEnv[p]) then
						return info.StructEnv[p]:Copy()
					end
				end
			end
		end

		------------------------------------
		--- Check if this first arg is a child class of the next arg
		-- @name System.Reflector.IsSuperClass
		-- @class function
		-- @param child the child class
		-- @param suepr the super class
		-- @return flag
		-- @usage System.Reflector.IsSuperClass(UIObject, Object)
		------------------------------------
		function IsSuperClass(child, super)
			if type(child) == "string" then child = ForName(child) end
			if type(super) == "string" then super = ForName(super) end

			return IsClass(child) and IsClass(super) and IsChildClass(super, child)
		end

		------------------------------------
		--- Get the class type of this object
		-- @name System.Reflector.GetObjectClass
		-- @class function
		-- @param obj the object
		-- @usage System.Reflector.GetObjectClass(obj)
		------------------------------------
		function GetObjectClass(obj)
			return type(obj) == "table" and getmetatable(obj) and getmetatable(obj).__class
		end

		------------------------------------
		--- Check if this object is an instance of the class
		-- @name System.Reflector.ObjectIsClass
		-- @class function
		-- @param obj the object
		-- @param class	the class type that you want to check if this object is an instance of.
		-- @usage System.Reflector.ObjectIsClass(obj, Object)
		------------------------------------
		function ObjectIsClass(obj, cls)
			if type(cls) == "string" then cls = ForName(cls) end
			return (obj and cls and IsChildClass(cls, GetObjectClass(obj))) or false
		end

		------------------------------------
		--- Active thread mode for special scripts.
		-- @name ActiveThread
		-- @class function
		-- @param obj object
		-- @param ... script name list
		-- @usage System.Reflector.ActiveThread(obj, "OnClick", "OnEnter")
		------------------------------------
		function ActiveThread(obj, ...)
			local cls = GetObjectClass(obj)
			local name

			if cls then
				for i = 1, select('#', ...) do
					name = select(i, ...)

					if HasScript(cls, name) then
						obj[name]._ThreadActived = true
					end
				end
			end
		end

		------------------------------------
		--- Whether the thread mode is actived for special scripts.
		-- @name IsThreadActived
		-- @class function
		-- @param obj object
		-- @param script
		-- @usage System.Reflector.IsThreadActived(obj, "OnClick")
		------------------------------------
		function IsThreadActived(obj, sc)
			local cls = GetObjectClass(obj)
			local name

			if cls and HasScript(cls, sc) then
				return obj[sc]._ThreadActived or false
			end

			return false
		end
		
		------------------------------------
		--- Inactive thread mode for special scripts.
		-- @name InactiveThread
		-- @class function
		-- @param obj object
		-- @param ... script name list
		-- @usage System.Reflector.InactiveThread(obj, "OnClick", "OnEnter")
		------------------------------------
		function InactiveThread(obj, ...)
			local cls = GetObjectClass(obj)
			local name

			if cls then
				for i = 1, select('#', ...) do
					name = select(i, ...)

					if HasScript(cls, name) then
						obj[name]._ThreadActived = nil
					end
				end
			end
		end

		------------------------------------
		--- Convert the object to a new class.
		-- @name ConvertClass
		-- @class function
		-- @param obj object
		-- @param cls class
		-- @usage System.Reflector.ConvertClass(obj, System.Addon)
		------------------------------------
		function ConvertClass(obj, ns)
			if type(obj) ~= "table" then return end

			if type(ns) == "string" then ns = ForName(ns) end

			local info = ns and _NSInfo[ns]

			if info and info.Type == TYPE_CLASS then
				local meta = getmetatable(obj)

				if meta ~= info.MetaTable then
					setmetatable(obj, info.MetaTable)
				end
			end
		end
	endclass "Reflector"
end

------------------------------------------------------
-- Global Settings
------------------------------------------------------
do
	--[[
	_G.class = class
	_G.enum = enum
	_G.namespace = namespace
	_G.struct = struct	--]]

	_Class_KeyWords.namespace = namespace
	_Class_KeyWords.class = class
	_Class_KeyWords.enum = enum
	_Class_KeyWords.struct = struct

	function IGAS:GetNameSpace(ns)
		return Reflector.ForName(ns)
	end
end
